Learn R Programming

propagate (version 1.1-0)

WelchSatter: Welch-Satterthwaite approximation to the 'effective degrees of freedom'

Description

Calculates the Welch-Satterthwaite approximation to the 'effective degrees of freedom' by using the samples' uncertainties and degrees of freedoms, as described in Welch (1947) and Satterthwaite (1946). External sensitivity coefficients can be supplied optionally.

Usage

WelchSatter(ui, ci = NULL, df = NULL, df.tot = NULL, uc = NULL, alpha = 0.05, k = NULL)

Value

A list with the following items:

ws.df

the 'effective degrees of freedom'.

k

the coverage factor for calculating the expanded uncertainty.

u.exp

the expanded uncertainty \(u_{\rm{exp}}\).

Arguments

ui

the uncertainties \(u_i\) for each variable \(x_i\).

ci

the sensitivity coefficients \(c_i = \partial y/\partial x_i\).

df

the degrees of freedom for the samples, \(\nu_i\).

df.tot

an optional known total degrees of freedom for the system, \(\nu_{\mathrm{tot}}\). Overrides the internal calculation of \(\nu_{\mathrm{ws}}\).

uc

the combined uncertainty, u(y).

alpha

the significance level for the t-statistic. See 'Details'.

k

an external coverage factor, say 2, that overrides the one obtained from defining alpha.

Author

Andrej-Nikolai Spiess

Details

$$\nu_{\rm{eff}} \approx \frac{u(y)^4}{\sum_{i = 1}^n \frac{(c_iu_i)^4}{\nu_i}}, \quad k = t(1 - (\alpha/2), \nu_{\rm{eff}}), \quad u_{\rm{exp}} = ku(y)$$

References

An Approximate Distribution of Estimates of Variance Components.
Satterthwaite FE.
Biometrics Bulletin (1946), 2: 110-114.

The generalization of "Student's" problem when several different population variances are involved.
Welch BL.
Biometrika (1947), 34: 28-35.

Examples

Run this code
## Taken from GUM H.1.6, 4).
WelchSatter(ui = c(25, 9.7, 2.9, 16.6), df = c(18, 25.6, 50, 2), uc = 32, alpha = 0.01)

Run the code above in your browser using DataLab