Learn R Programming

propagate (version 1.1-0)

plot.propagate: Plotting function for 'propagate' objects

Description

Creates a histogram (blue) of the evaluated results from the multivariate simulated data, along with a density curve (red), \(\alpha\)-based confidence intervals (GUM: dashed black; MC: solid black), median (orange) and mean (green).

Usage

# S3 method for propagate
plot(x, logx = FALSE, breaks = 100, ...)

Value

A plot as described above.

Arguments

x

an object returned from propagate.

logx

logical. Should the data be displayed on a logarithmic abscissa?

breaks

numeric. The number of bins for the histogram.

...

other parameters to hist.

Author

Andrej-Nikolai Spiess

Examples

Run this code
EXPR1 <- expression(x^2 * sin(y))
x <- c(5, 0.01)
y <- c(1, 0.01)
DF1 <- cbind(x, y)
RES1 <- propagate(expr = EXPR1, data = DF1, type = "stat", 
                  nsim = 100000, alpha = 0.01)
plot(RES1)

Run the code above in your browser using DataLab